#! /usr/bin/python3
from dataclasses import dataclass
from math import sqrt
from typing import List

@dataclass
class Point:
    x: float
    y: float

    def __sub__(self, p):
        return Point(self.x - p.x, self.y - p.y)

@dataclass
class Circle(Point):
    r: float

def find_external_tangent_points(c1: Circle, c2: Circle) -> List[Tuple[Point, Point]]:
    # Tính khoảng cách giữa hai tâm
    d = sqrt((c2.x - c1.x)**2 + (c2.y - c1.y)**2)
    
    # Góc của đường nối tâm
    base_angle = atan2(c2.y - c1.y, c2.x - c1.x)

    # Góc của tiếp tuyến ngoài
    angle = atan2(c2.r - c1.r, d)

    tangent_points = []
    for delta in [angle, -angle]:
        angle1 = base_angle + delta
        angle2 = base_angle + delta + 3.14159  # Cộng thêm π để tính điểm đối diện

        x_t1 = c1.x + c1.r * cos(angle1)
        y_t1 = c1.y + c1.r * sin(angle1)
        x_t2 = c2.x + c2.r * cos(angle2)
        y_t2 = c2.y + c2.r * sin(angle2)
        tangent_points.append((Point(x_t1, y_t1), Point(x_t2, y_t2)))

    return tangent_points

# Ví dụ
c1 = Circle(111.9, 18.900000000000002, 0.9009009009009009)
c2 = Circle(111.9, 20.400000000000002, 0.9009009009009009)
tangent_points = find_external_tangent_points(c1, c2)

for i, (p1, p2) in enumerate(tangent_points, start=1):
    print(f"Tangent {i}: Circle 1 at ({p1.x}, {p1.y}), Circle 2 at ({p2.x}, {p2.y})")
